package com.care.bigscreen.controller;


import com.care.bigscreen.service.BigScreenService;
import com.care.bigscreen.vo.*;

import com.care.common.vo.PageResVO;
import com.care.common.util.PageResult;
import com.care.common.util.PageUtil;
import com.care.common.util.Result;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @author javadoc
 * @version 1.0
 * @description 大屏控制器
 * @date
 */
@Slf4j
@RestController
@RequestMapping("/bms/bigscreen")
public class BigScreenController {

    @Autowired
    private BigScreenService bigScreenService;

    /////////////////////////////////////////////////////////运营商大屏////////////////////////////////////////////////////
    /**
     * 运营商大屏统计查询
     */
    @ApiOperation(value = "运营商大屏统计查询", notes = "运营商大屏统计查询")
    @GetMapping("/selectBigScreenStatisticsByOp")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息")
    })
    public Result<OpBigScreenStatisticsVO> selectBigScreenStatisticsByOp(@RequestHeader("token") String token) {
        try {
            OpBigScreenStatisticsVO opBigScreenStatisticsVO =  bigScreenService.selectBigScreenStatisticsByOp(null);
            return Result.success(opBigScreenStatisticsVO);
        } catch (Exception e) {
           log.error("运营商大屏统计查询出现异常",e);
            return Result.error("系统错误，运营商大屏统计查询失败");
        }
    }


    /**
     * 运营商大屏的站点统计查询
     */
    @ApiOperation(value = "运营商大屏的站点统计查询", notes = "运营商大屏的站点统计查询")
    @GetMapping("/selectOpBigScreenStatisticsByStation")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<OpBigScreenStatisticsVO> selectOpBigScreenStatisticsByStation(@RequestHeader("token") String token, Long stationId) {
        try {
            OpBigScreenStatisticsVO opBigScreenStatisticsVO =  bigScreenService.selectBigScreenStatisticsByOp(stationId);
            return Result.success(opBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("运营商大屏的站点统计查询出现异常",e);
            return Result.error("系统错误，运营商大屏的站点统计查询失败");
        }
    }

    /**
     * 运营商大屏的根据经纬度查询统计值
     */
    @ApiOperation(value = "运营商大屏的根据经纬度查询统计值", notes = "运营商大屏的根据经纬度查询统计值")
    @GetMapping("/selectOpBigScreenStatisticsByLongitudeLatitude")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "longitudeLeftUp", value = "左上经度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "latitudeLeftUp", value = "左上维度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "longitudeRightDown", value = "右下经度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "latitudeRightDown", value = "右下维度", dataTypeClass = String.class,required = true)
            }
    )
    public Result<OpBigScreenStatisticsVO> selectOpBigScreenStatisticsByLongitudeLatitude(@RequestHeader("token") String token, String longitudeLeftUp, String latitudeLeftUp, String longitudeRightDown, String latitudeRightDown) {
        try {
            OpBigScreenStatisticsVO opBigScreenStatisticsVO =  bigScreenService.selectOpBigScreenStatisticsByLongitudeLatitude(longitudeLeftUp,latitudeLeftUp,longitudeRightDown,latitudeRightDown);
            return Result.success(opBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("运营商大屏或机构大屏的根据经纬度查询统计值出现异常",e);
            return Result.error("系统错误，运营商大屏或机构大屏的根据经纬度查询统计值失败");
        }
    }

    /**
     * 运营商大屏的站点列表查询
     */
    @ApiOperation(value = "运营商大屏的站点列表查询", notes = "运营商大屏的站点列表查询")
    @GetMapping("/selectOpBigScreenStationList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息")
            }
    )
    public Result<List<BigScreenStationVO>> selectOpBigScreenStationList(@RequestHeader("token") String token) {
        try {
            List<BigScreenStationVO> bigScreenStationVOList =  bigScreenService.selectBigScreenStationList(null);
            return Result.success(bigScreenStationVOList);
        } catch (Exception e) {
            log.error("运营商大屏的站点列表查询出现异常",e);
            return Result.error("系统错误，运营商大屏的站点列表查询失败");
        }
    }

    /////////////////////////////////////////////////////////机构大屏////////////////////////////////////////////////////
    /**
     * 机构大屏统计查询
     */
    @ApiOperation(value = "机构大屏统计查询", notes = "机构大屏统计查询")
    @GetMapping("/selectBigScreenStatisticsByOrg")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "orgId", value = "机构ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<OrgBigScreenStatisticsVO> selectBigScreenStatisticsByOrg(@RequestHeader("token") String token, Long orgId) {
        try {
            OrgBigScreenStatisticsVO orgBigScreenStatisticsVO =  bigScreenService.selectBigScreenStatisticsByOrg(null,orgId);
            return Result.success(orgBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("机构大屏统计查询异常",e);
            return Result.error("系统错误，机构大屏统计查询失败");
        }
    }
    /**
     * 机构大屏的站点统计查询
     */
    @ApiOperation(value = "机构大屏的站点统计查询", notes = "机构大屏的站点统计查询")
    @GetMapping("/selectOrgBigScreenStatisticsByStation")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<OrgBigScreenStatisticsVO> selectOrgBigScreenStatisticsByStation(@RequestHeader("token") String token, Long stationId) {
        try {
            OrgBigScreenStatisticsVO orgBigScreenStatisticsVO =  bigScreenService.selectBigScreenStatisticsByOrg(stationId,null);
            return Result.success(orgBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("运营商大屏或机构大屏的站点统计查询出现异常",e);
            return Result.error("系统错误，运营商大屏或机构大屏的站点统计查询失败");
        }
    }

    /**
     * 机构大屏的根据经纬度查询统计值
     */
    @ApiOperation(value = "机构大屏的根据经纬度查询统计值", notes = "机构大屏的根据经纬度查询统计值")
    @GetMapping("/selectOrgBigScreenStatisticsByLongitudeLatitude")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "longitudeLeftUp", value = "左上经度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "latitudeLeftUp", value = "左上维度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "longitudeRightDown", value = "右下经度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "latitudeRightDown", value = "右下维度", dataTypeClass = String.class,required = true),
                    @ApiImplicitParam(name = "orgId", value = "机构ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<OrgBigScreenStatisticsVO> selectOrgBigScreenStatisticsByLongitudeLatitude(@RequestHeader("token") String token, String longitudeLeftUp, String latitudeLeftUp, String longitudeRightDown, String latitudeRightDown, Long orgId) {
        try {
            OrgBigScreenStatisticsVO orgBigScreenStatisticsVO =  bigScreenService.selectOrgBigScreenStatisticsByLongitudeLatitude(longitudeLeftUp,latitudeLeftUp,longitudeRightDown,latitudeRightDown,orgId);
            return Result.success(orgBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("机构大屏的根据经纬度查询统计值出现异常",e);
            return Result.error("系统错误，机构大屏的根据经纬度查询统计值失败");
        }
    }
    /**
     * 机构大屏的站点列表查询
     */
    @ApiOperation(value = "机构大屏的站点列表查询", notes = "机构大屏的站点列表查询")
    @GetMapping("/selectOrgBigScreenStationList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "orgId", value = "机构ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<List<BigScreenStationVO>> selectOrgBigScreenStationList(@RequestHeader("token") String token,Long orgId) {
        try {
            List<BigScreenStationVO> bigScreenStationVOList =  bigScreenService.selectBigScreenStationList(orgId);
            return Result.success(bigScreenStationVOList);
        } catch (Exception e) {
            log.error("机构大屏的站点列表查询出现异常",e);
            return Result.error("系统错误，机构大屏的站点列表查询失败");
        }
    }

    /**
     * 机构大屏的进行中的安全事件列表查询
     */
    @ApiOperation(value = "机构大屏的进行中的安全事件列表查询", notes = "机构大屏的进行中的安全事件列表查询")
    @GetMapping("/selectOrgBigScreenRtEventList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "orgId", value = "机构ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<List<BigScreenEventVO>> selectOrgBigScreenRtEventList(@RequestHeader("token") String token,Long orgId) {
        try {
            List<BigScreenEventVO> bigScreenEventVOList =  bigScreenService.selectBigScreenRtEventList(orgId,null);
            return Result.success(bigScreenEventVOList);
        } catch (Exception e) {
            log.error("机构大屏的进行中的安全事件列表查询出现异常",e);
            return Result.error("系统错误，机构大屏的进行中的安全事件列表查询失败");
        }
    }

    /**
     * 机构大屏的设备列表查询
     */
    @ApiOperation(value = "机构大屏的设备列表查询", notes = "机构大屏的设备列表查询")
    @GetMapping("/selectOrgBigScreenDeviceList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "orgId", value = "机构ID", dataTypeClass = Long.class,required = true),
                    @ApiImplicitParam(name = "stationId", value = "服务站ID", dataTypeClass = Long.class,required = false),
                    @ApiImplicitParam(name = "status", value = "状态：1在线；2：离线", dataTypeClass = String.class,required = false)
            }
    )
    public Result<List<BigScreenDeviceVO>> selectOrgBigScreenDeviceList(@RequestHeader("token") String token,Long orgId,Long stationId,String status) {
        try {
            List<BigScreenDeviceVO> bigScreenDeviceVOList =  bigScreenService.selectOrgBigScreenDeviceList(orgId,stationId,status);
            return Result.success(bigScreenDeviceVOList);
        } catch (Exception e) {
            log.error("机构大屏的设备列表查询出现异常",e);
            return Result.error("系统错误，机构大屏的设备列表查询失败");
        }
    }

    /////////////////////////////////////////////////////////服务站大屏////////////////////////////////////////////////////
    /**
     * 服务站大屏的站点统计查询
     */
    @ApiOperation(value = "服务站大屏的站点统计查询", notes = "服务站大屏的站点统计查询")
    @GetMapping("/selectStationBigScreenStatisticsByStation")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<StationBigScreenStatisticsVO> selectStationBigScreenStatisticsByStation(@RequestHeader("token") String token, Long stationId) {
        try {
            StationBigScreenStatisticsVO stationBigScreenStatisticsVO =  bigScreenService.selectStationBigScreenStatisticsByStation(stationId);
            return Result.success(stationBigScreenStatisticsVO);
        } catch (Exception e) {
            log.error("服务站大屏的站点统计查询出现异常",e);
            return Result.error("系统错误，服务站大屏的站点统计查询失败");
        }
    }


    /**
     * 服务站大屏的告警用户列表查询
     */
    @ApiOperation(value = "服务站大屏的告警用户列表查询", notes = "服务站大屏的告警用户列表查询")
    @GetMapping("/selectBigScreenAlarmOlderList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true),
                    @ApiImplicitParam(name = "olderName", value = "被监护人姓名", dataTypeClass = String.class,required = false)
            }
    )
    public Result<List<BigScreenAlarmOlderVO>> selectBigScreenAlarmOlderList(@RequestHeader("token") String token,Long stationId,String olderName) {
        try {
            List<BigScreenAlarmOlderVO> bigScreenAlarmOlderVOList =  bigScreenService.selectBigScreenAlarmOlderList(stationId,olderName);
            return Result.success(bigScreenAlarmOlderVOList);
        } catch (Exception e) {
            log.error("告警用户列表查询出现异常",e);
            return Result.error("系统错误，告警用户列表查询失败");
        }
    }


    /**
     * 服务站大屏的被监控人详细查询
     */
    @ApiOperation(value = "服务站大屏的被监控人详细查询", notes = "服务站大屏的被监控人详细查询")
    @GetMapping("/selectStationBigScreenOlderDetail")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "olderId", value = "被监护人ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<BigScreenOlderDetailVO> selectStationBigScreenOlderDetail(@RequestHeader("token") String token,Long olderId) {
        try {
            BigScreenOlderDetailVO bigScreenOlderDetailVO =  bigScreenService.selectStationBigScreenOlderDetail(olderId);
            return Result.success(bigScreenOlderDetailVO);
        } catch (Exception e) {
            log.error("服务站大屏的被监控人详细查询出现异常",e);
            return Result.error("系统错误，服务站大屏的被监控人详细查询失败");
        }
    }

    /**
     * 服务站大屏的安全事件总计查询
     */
    @ApiOperation(value = "服务站大屏的安全事件总计查询", notes = "服务站大屏的安全事件总计查询")
    @GetMapping("/selectStationBigScreenEventTotal")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<BigScreenEventTotalVO> selectStationBigScreenEventTotal(@RequestHeader("token") String token,Long stationId) {
        try {
            BigScreenEventTotalVO bigScreenEventTotalVO =  bigScreenService.selectStationBigScreenEventTotal(stationId);
            return Result.success(bigScreenEventTotalVO);
        } catch (Exception e) {
            log.error("服务站大屏的安全事件总计查询出现异常",e);
            return Result.error("系统错误，服务站大屏的安全事件总计查询失败");
        }
    }

    /**
     * 服务站大屏的实时安全事件列表查询
     */
    @ApiOperation(value = "服务站大屏的实时安全事件列表查询", notes = "服务站大屏的实时安全事件列表查询")
    @GetMapping("/selectBigScreenRtEventList")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<List<BigScreenEventVO>> selectBigScreenRtEventList(@RequestHeader("token") String token,Long stationId) {
        try {
            List<BigScreenEventVO> bigScreenEventVOList =  bigScreenService.selectBigScreenRtEventList(null,stationId);
            return Result.success(bigScreenEventVOList);
        } catch (Exception e) {
            log.error("服务站大屏的实时安全事件列表查询出现异常",e);
            return Result.error("系统错误，服务站大屏的实时安全事件列表查询失败");
        }
    }

    /**
     * 服务站大屏的已完成安全事件列表分页查询
     */
    @ApiOperation(value = "服务站大屏的已完成安全事件列表分页查询", notes = "服务站大屏的已完成安全事件列表分页查询")
    @GetMapping("/selectBigScreenHisEventListByPage")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public PageResult<List<BigScreenEventVO>> selectBigScreenHisEventListByPage(@RequestHeader("token") String token,Long stationId, PageResVO page) {
        try {
            PageUtil<BigScreenEventVO> bigScreenEventVOListByPage =  bigScreenService.selectBigScreenHisEventListByPage(stationId,page);
            return  PageResult.success(
                    "查询成功",bigScreenEventVOListByPage.getListT(),
                    bigScreenEventVOListByPage.getCurrent(),
                    bigScreenEventVOListByPage.getPageSize(),
                    bigScreenEventVOListByPage.getTotal());
        } catch (Exception e) {
            log.error("服务站大屏的已完成安全事件列表分页查询出现异常",e);
            return PageResult.error("系统错误，服务站大屏的已完成安全事件列表分页查询失败");
        }
    }

    /**
     * 服务站大屏的用户卡片总计查询
     */
    @ApiOperation(value = "服务站大屏的用户卡片总计查询", notes = "服务站大屏的用户卡片总计查询")
    @GetMapping("/selectStationBigScreenOlderCardTotal")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "olderName", value = "被监护人姓名", dataTypeClass = String.class,required = false),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true)
            }
    )
    public Result<BigScreenOlderCardTotalVO> selectStationBigScreenOlderCardTotal(@RequestHeader("token") String token,Long stationId,String olderName) {
        try {
            BigScreenOlderCardTotalVO bigScreenOlderCardTotalVO =  bigScreenService.selectStationBigScreenOlderCardTotal(stationId,olderName);
            return Result.success(bigScreenOlderCardTotalVO);
        } catch (Exception e) {
            log.error("服务站大屏的用户卡片总计查询出现异常",e);
            return Result.error("系统错误，服务站大屏的用户卡片总计查询失败");
        }
    }

    /**
     * 服务站大屏的用户卡片列表分页查询
     */
    @ApiOperation(value = "服务站大屏的用户卡片列表分页查询", notes = "服务站大屏的用户卡片列表分页查询")
    @GetMapping("/selectBigScreenOlderCardListByPage")
    @ApiImplicitParams(
            {
                    @ApiImplicitParam(paramType = "header", required = true, name = "token", dataType = "String", value = "token验证信息"),
                    @ApiImplicitParam(name = "stationId", value = "站点ID", dataTypeClass = Long.class,required = true),
                    @ApiImplicitParam(name = "olderName", value = "被监护人姓名", dataTypeClass = String.class,required = false),
                    @ApiImplicitParam(name = "isAlarm", value = "是告警：1，不是告警（全部）：0", dataTypeClass = String.class,required = false)
            }
    )
    public PageResult<List<BigScreenOlderDetailVO>> selectBigScreenOlderCardListByPage(@RequestHeader("token") String token,Long stationId,String olderName,String isAlarm, PageResVO page) {
        try {
            PageUtil<BigScreenOlderDetailVO> bigScreenOlderDetailVOList =  bigScreenService.selectBigScreenOlderCardListByPage(stationId,olderName,isAlarm,page);
            return  PageResult.success(
                    "查询成功",bigScreenOlderDetailVOList.getListT(),
                    bigScreenOlderDetailVOList.getCurrent(),
                    bigScreenOlderDetailVOList.getPageSize(),
                    bigScreenOlderDetailVOList.getTotal());
        } catch (Exception e) {
            log.error("服务站大屏的用户卡片列表分页查询出现异常",e);
            return PageResult.error("系统错误，服务站大屏的用户卡片列表分页查询失败");
        }
    }
}
