package com.care.bigscreen.service;

import com.care.bigscreen.vo.*;
import com.care.common.vo.PageResVO;
import com.care.common.util.PageUtil;

import java.io.IOException;
import java.util.List;


/**
 * @author stw
 * @version 1.0
 * @description 站点统计表接口实现类
 * @date 2021/05/20/19:41
 */
public interface BigScreenService {
    /**
     * 运营商大屏统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    OpBigScreenStatisticsVO selectBigScreenStatisticsByOp(Long stationId);

    /**
     * 运营商大屏根据经纬度统计查询
     * @return 根据经纬度范围搜索符合条件的大屏统计数据
     */
    OpBigScreenStatisticsVO selectOpBigScreenStatisticsByLongitudeLatitude(String longitudeLeftUp, String latitudeLeftUp, String longitudeRightDown, String latitudeRightDown);

    /**
     * 机构大屏统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    OrgBigScreenStatisticsVO selectBigScreenStatisticsByOrg(Long stationId, Long orgId);


    /**
     * 运营商大屏根据经纬度统计查询
     * @return 根据经纬度范围搜索符合条件的大屏统计数据
     */
    OrgBigScreenStatisticsVO selectOrgBigScreenStatisticsByLongitudeLatitude(String longitudeLeftUp, String latitudeLeftUp, String longitudeRightDown, String latitudeRightDown, Long orgId);

    /**
     * 站点列表查询
     */
    List<BigScreenStationVO> selectBigScreenStationList(Long orgId);


    /**
     * 服务站大屏的站点统计查询
     * @return 搜索符合条件的大屏统计数据
     */
    StationBigScreenStatisticsVO selectStationBigScreenStatisticsByStation(Long stationId);

    /**
     * 告警用户列表查询
     */
    List<BigScreenAlarmOlderVO> selectBigScreenAlarmOlderList(Long stationId,String olderName);

    /**
     * 服务站大屏的被监控人详细查询
     */
    BigScreenOlderDetailVO selectStationBigScreenOlderDetail(Long olderId);

    /**
     * 服务站大屏的事件总计查询
     */
    BigScreenEventTotalVO selectStationBigScreenEventTotal(Long stationId);

    /**
     * 机构大屏或服务站大屏的实时安全事件列表查询
     */
    List<BigScreenEventVO> selectBigScreenRtEventList(Long orgId, Long stationId);

    /**
     * 机构大屏的设备列表查询
     */
    List<BigScreenDeviceVO> selectOrgBigScreenDeviceList(Long orgId, Long stationId,String status);

    /**
     * 服务站大屏的已完成安全事件列表分页查询
     */
    PageUtil<BigScreenEventVO> selectBigScreenHisEventListByPage(Long stationId, PageResVO page);

    /**
     * 服务站大屏的用户卡片总计查询
     */
    BigScreenOlderCardTotalVO selectStationBigScreenOlderCardTotal(Long stationId,String olderName);

    /**
     * 服务站大屏的用户卡片列表分页查询
     */
    PageUtil<BigScreenOlderDetailVO> selectBigScreenOlderCardListByPage(Long stationId, String olderName,String isAlarm, PageResVO page);

    /**
     * 调用websocket推送实时事件标识给前端
     */
    void pushRtEventFlag(Long orgId,Long stationId,String orderId,String flag);


    /**
     * 进行统计
     */
    void exeStatistics();
}