package com.care.bms.controller;

import cn.hutool.json.JSONUtil;
import com.care.bms.params.UserLoginParams;
import com.care.bms.service.BmsSysUserService;
import com.care.common.constant.Constants;
import com.care.common.exception.BDException;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.util.JwtUtils;
import com.care.common.util.Result;
import com.care.common.util.ResultCode;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by y on 2017/10/20.
 * @author y
 */
@RestController
@Api(value = "LoginController", tags = { "登录相关接口" })
@Slf4j
@RequestMapping("/bms")
public class LoginController {

    @Autowired
    BmsSysUserService careSysUserService;

    @PostMapping(value = "/login", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "登陆", notes = "登陆--二期修改")
    public Result<Object> login(HttpServletRequest request, @RequestBody UserLoginParams userLoginParams) {
        try {
            //验证用户名密码
            UserLogindConvertVO loginUser = careSysUserService.getLoginUser(userLoginParams.getOrgCode(),userLoginParams.getPhone(),userLoginParams.getPassword());
            if (loginUser == null) {
                return Result.error("未找到相关用户");
            }
            //todo ： 放入到 redis 中
            Map<String,Object> dataMap = new HashMap<>(2);
            dataMap.put("token",JwtUtils.generalToken(loginUser));
            dataMap.put("user",loginUser);
            return Result.success("登陆成功",dataMap);

        }catch (BDException ex) {
            return Result.error(ex.getMessage());
        } catch (Exception ex) {
            return new Result(ResultCode.NO_LOGINED,"登陆失败",ex.getMessage());
        }
    }

    @GetMapping(value = "/parseToken", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "解析token", notes = "解析token")
    public Result<Object> tokenParse(HttpServletRequest request) {

        String commonJwt = request.getHeader(Constants.LOGIN_TOKEN_KEY);
        Claims claims = JwtUtils.tokenParse(commonJwt);
        if (claims != null) {
            UserLogindConvertVO vo = JSONUtil.toBean(claims.getSubject(), UserLogindConvertVO.class);
            if (vo != null) {
               return Result.success("解析成功",vo);
            }else{
                return Result.error("解析失败");
            }
        }
        return Result.error("解析失败");
    }
    //TODO 退出
}
