package com.care.client.controller;

import com.care.client.service.PassportService;
import com.care.client.service.PinanbaoDeviceService;
import com.care.client.vo.HouseContactVO;
import com.care.client.vo.MemberInfoVO;
import com.care.client.vo.DeviceVO;
import com.care.client.vo.OlderVO;
import com.care.common.exception.BDException;
import com.care.common.util.Result;
import com.care.common.vo.device.StationVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "DevApiController", tags = { "我的设备" })
@Slf4j
@RequestMapping("/pinanbao/device")
public class DevApiController {

    @Resource
    private PinanbaoDeviceService pinanbaoDeviceService;
    @Resource
    private PassportService passportService;

    /**
     * 我的设备列表
     * @return
     */
    @GetMapping("/my/myDeviceList")
    @ApiOperation(tags = {"我的设备"},value = "我的设备列表 -- 二期新增")
    public Result<List<DeviceVO>> myDeviceList(@RequestHeader(value = "token") String token){
        try {
            MemberInfoVO current = passportService.checkToken(token);
            List<DeviceVO> datas = pinanbaoDeviceService.queryDeviceByMemberId(current.getId());
            return Result.success(datas);
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("设备: 列表查询出现异常",e);
            return Result.error( "获取失败");
        }
    }



    @PostMapping("/my/unBindMyDevice")
    @ApiOperation(tags = {"我的设备"},value = "在设备列表中移除设备（解除绑定） -- 二期新增")
    public Result<Object> unBindMyDevice(HttpServletRequest request,@RequestHeader("token") String token,
                                         @RequestParam("devId") Long devId) {
        try {
            MemberInfoVO current = passportService.checkToken(token);
            this.pinanbaoDeviceService.unBindMyDevice(current.getId(),devId);
            return Result.success("移除成功！");
        } catch (Exception e) {
            log.error("设备: 移除设备出现异常", e);
            return Result.error("移除失败！");
        }
    }



    /**
     * 我的设备详情
     * @return
     */
    @GetMapping("/my/getMyDeviceInfo")
    @ApiOperation(tags = {"我的设备"},value = "设备详情 （包括设备名、场景、响应时间、服务信息，是否设置xxx的标志等） -- 二期新增")
    public Result<DeviceVO> getMyDeviceInfo(@RequestHeader(value = "token") String token,
                                                @RequestParam("devId") Long devId){
        try {
            return Result.success(this.pinanbaoDeviceService.getDeviceInfo(devId));
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("设备: 查询详情出现异常",e);
            return Result.error( "获取失败");
        }
    }

    /**
     * 我的设备-服务站简介
     * @return
     */
    @GetMapping("/my/getMyDevStationInfo")
    @ApiOperation(tags = {"我的设备"},value = "服务站简介 -- 二期新增")
    public Result<StationVO> getMyDevStationInfo(@RequestHeader(value = "token") String token,
                                                 @RequestParam("stationId") Long stationId){
        try {
            return Result.success(this.pinanbaoDeviceService.getMyDevStationInfo(stationId));
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站: 查询详情出现异常",e);
            return Result.error( "获取失败");
        }
    }

    @PostMapping("/my/updateMyDevice")
    @ApiOperation(tags = {"我的设备"},value = "设备修改(设备名，场景、响应时间) -- 二期新增")
    public Result<Object> updateMyDevice(@RequestHeader("token") String token,
                                 @RequestBody DeviceVO vo){
        try {
            if(this.pinanbaoDeviceService.updateMyDevice(vo)){
                return  Result.success("修改成功！");
            } else {
                return Result.error("修改失败！");
            }

        }catch (BDException e) {
            log.error("修改设备-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("设备: 修改设备出现异常",e);
            return Result.error("修改失败！");
        }
    }


    /**
     * 查询我的设备拥有的被监护人列表
     * @return
     */
    @GetMapping("/my/queryMyOlderListByDeviceId")
    @ApiOperation(tags = {"我的设备"},value = "查询我的设备拥有的被监护人列表 -- 二期新增")
    public Result<List<OlderVO>> queryMyOlderListByDeviceId(@RequestHeader(value = "token") String token,
                                                   @RequestParam("devId") Long devId){
        try {
            List<OlderVO> datas = pinanbaoDeviceService.queryMyOlderListByDeviceId(devId);
            return Result.success(datas);
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("老人: 列表查询出现异常",e);
            return Result.error( "获取失败");
        }
    }


    /**
     * 查询我的设备拥有的紧急联系人列表
     * @return
     */
    @GetMapping("/my/queryMyContactListByDeviceId")
    @ApiOperation(tags = {"我的设备"},value = "查询我的设备拥有的紧急联系人列表 -- 二期新增")
    public Result<List<HouseContactVO>> queryMyContactListByDeviceId(@RequestHeader(value = "token") String token,
                                                            @RequestParam("devId") Long devId){
        try {
            List<HouseContactVO> datas = pinanbaoDeviceService.queryMyContactListByDeviceId(devId);
            return Result.success(datas);
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("联系人: 列表查询出现异常",e);
            return Result.error( "获取失败");
        }
    }

    /**
     * 选择我的被监护人列表中某个老人保存绑定
     * @return
     */
    @PostMapping("/my/bindOlder")
    @ApiOperation(tags = {"我的设备"},value = "选择我的被监护人列表中某个老人保存绑定 --二期新增")
    public Result<Object> bindOlder(HttpServletRequest request, @RequestHeader("token") String token,
                                    @RequestParam("devId") Long devId,
                                     @RequestParam("olderId") Long olderId) {
        if (this.pinanbaoDeviceService.bindOlder(devId,olderId)) {
            return  Result.success("绑定成功");
        } else{
            return  Result.error("绑定失败");
        }
    }

    @PostMapping("/my/unBindOlder")
    @ApiOperation(tags = {"我的设备"},value = "移除被监护人（解除绑定） -- 二期新增")
    public Result<Object> unBindOlder(HttpServletRequest request,@RequestHeader("token") String token,
                                      @RequestParam("devId") Long devId,
                                      @RequestParam("olderId") Long olderId) {
        try {
            this.pinanbaoDeviceService.unBindOlder(devId,olderId);
            return Result.success("移除成功！");
        } catch (Exception e) {
            log.error("设备: 移除设备出现异常", e);
            return Result.error("移除失败！");
        }
    }
    /**
     * 选择我的紧急人列表中某个保存绑定
     * @return
     */
    @PostMapping("/my/bindHouseContact")
    @ApiOperation(tags = {"我的设备"},value = "选择我的紧急人列表中某个保存绑定 --二期新增")
    public Result<Object> bindHouseContact(HttpServletRequest request, @RequestHeader("token") String token,
                                    @RequestParam("devId") Long devId,
                                    @RequestParam("contactId") Long contactId) {
        if (this.pinanbaoDeviceService.bindHouseContact(devId,contactId)) {
            return  Result.success("绑定成功");
        } else{
            return  Result.error("绑定失败");
        }
    }

    /**
     * 移除被紧急联系人（解除绑定）
     * @return
     */
    @PostMapping("/my/unHouseContact")
    @ApiOperation(tags = {"我的设备"},value = "移除被紧急联系人（解除绑定） --二期新增")
    public Result<Object> unHouseContact(HttpServletRequest request, @RequestHeader("token") String token,
                                           @RequestParam("devId") Long devId,
                                           @RequestParam("contactId") Long contactId) {
        if (this.pinanbaoDeviceService.unHouseContact(devId,contactId)) {
            return  Result.success("绑定成功");
        } else{
            return  Result.error("绑定失败");
        }
    }



}
