package com.care.client.controller;

import com.care.client.service.PassportService;
import com.care.client.service.PinanbaoOlderService;
import com.care.client.vo.MemberInfoVO;
import com.care.client.vo.OlderVO;
import com.care.common.exception.BDException;
import com.care.common.util.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "OlderApiController", tags = { "我的被监护人" })
@Slf4j
@RequestMapping("/pinanbao/older")
public class OlderApiController {

    @Resource
    private PinanbaoOlderService pinanbaoOlderService;
    @Resource
    private PassportService passportService;

    /**
     * 我的被监护人列表
     * @return
     */
    @GetMapping("/my/list")
    @ApiOperation(tags = {"我的被监护人,我的设备"},value = "被监护人列表")
    public Result<List<OlderVO>> myOlderList(@RequestHeader(value = "token") String token){
        try {
            MemberInfoVO current = passportService.checkToken(token);
            List<OlderVO> datas = pinanbaoOlderService.queryOlderByMemberId(current.getId());
            return Result.success(datas);
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("被监护人: 列表查询出现异常",e);
            return Result.error( "获取失败");
        }
    }

    /**
     * 我的被监护人详情
     * @return
     */
    @GetMapping("/my/getMyOlderInfo")
    @ApiOperation(tags = {"我的被监护人"},value = "被监护人详情  -- 二期新增")
    public Result<OlderVO> getMyOlderInfo(@RequestHeader(value = "token") String token,
                                                @RequestParam("olderId") Long olderId){
        try {
            return Result.success(this.pinanbaoOlderService.getOlderInfo(olderId));
        }catch (BDException e) {
            return  Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("被监护人: 查询详情出现异常",e);
            return Result.error( "获取失败");
        }
    }


    @PostMapping("/my/add")
    @ApiOperation(tags = {"我的被监护人,我的设备"},value = "被监护人新增 -- 二期新增")
    public Result<Object> addMyOlder(@RequestHeader("token") String token,
                                   @RequestBody OlderVO vo){
        try {
            MemberInfoVO current = passportService.checkToken(token);
            if(this.pinanbaoOlderService.createMyOlder(current.getId(),vo)){
                return  Result.success("新增成功！");
            } else {
                return Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增被监护人-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("被监护人: 新增被监护人出现异常",e);
            return Result.error("新增失败！");
        }
    }

    @PostMapping("/my/update")
    @ApiOperation(tags = {"我的被监护人,我的设备"},value = "被监护人修改 -- 二期新增")
    public Result<Object> updateMyOlder(@RequestHeader("token") String token,
                                 @RequestBody OlderVO vo){
        try {
            if(this.pinanbaoOlderService.updateMyOlder(vo)){
                return  Result.success("修改成功！");
            } else {
                return Result.error("修改失败！");
            }

        }catch (BDException e) {
            log.error("修改被监护人-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("被监护人: 修改被监护人出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @DeleteMapping("/my/delete/{olderId}")
    @ApiOperation(tags = {"我的被监护人"},value = "删除被监护人 -- 二期新增")
    public Result<Object> deleteMyOlder(HttpServletRequest request,@RequestHeader("token") String token,
                                 @PathVariable("olderId") Long olderId) {
        try {
            this.pinanbaoOlderService.deleteById(olderId);
            return Result.success("删除成功！");
        } catch (Exception e) {
            log.error("被监护人: 删除被监护人出现异常", e);
            return Result.error("删除失败！");
        }
    }
}
