package com.care.client.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.care.client.vo.DeviceVO;
import com.care.client.vo.HouseContactVO;
import com.care.client.vo.OlderVO;
import com.care.common.entity.*;
import com.care.common.enums.*;
import com.care.common.service.*;
import com.care.common.vo.device.StationVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @Author: lilt
 * @Date: 2021/6/7
 * @Desc:
 */
@Slf4j
@Service
public class PinanbaoDeviceService {

    @Resource
    private CareMemberInfoService careMemberInfoService;
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareSysUserService careSysUserService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareHouseOlderRelService careHouseOlderRelService;
    @Resource
    private CareOlderService careOlderService;
    @Resource
    private CareHouseContactRelService careHouseContactRelService;
    @Resource
    private CareHouseContactService careHouseContactService;


    /**
     * 移除设备
     * @return
     */
    public Boolean unBindMyDevice(Long memberId,Long devId) {
        UpdateWrapper<CareDevice> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CareDevice::getMemberId,memberId).eq(CareDevice::getId,devId)
        .set(CareDevice::getMemberId,null).set(CareDevice::getMemberPhone,null);
        return careDeviceService.update(updateWrapper);
    }

    /**
     * 查询设备列表
     * @param memberId
     * @return
     */
    public List<DeviceVO> queryDeviceByMemberId(Long memberId){
        List<DeviceVO> vos = Lists.newArrayList();
        CareMemberInfo memberInfo = this.careMemberInfoService.getById(memberId);
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getMemberId, memberInfo.getId());
        List<CareDevice> devices = this.careDeviceService.list(queryWrapper);
        if (CollUtil.isNotEmpty(devices)){
            devices.forEach(item -> {
                DeviceVO vo = new DeviceVO();
                BeanUtil.copyProperties(item,vo);
                vo.setDevSceneName(DeviceSceneEnum.getCodeToName(vo.getDevScene()));
                vo.setActiveStatusName(DeviceActiveStatusEnum.getCodeToName(vo.getActiveStatus()));
                vos.add(vo);
            });
        }
        return vos;
    }

    /**
     * 新增设备
     * @param memberId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean createMyDevice(Long memberId,DeviceVO vo){
        //保存被监护人
        CareDevice careDevice = new CareDevice();
        BeanUtil.copyProperties(vo,careDevice);
        careDevice.setMemberId(memberId);
        careDevice.setCreateTime(new Date());
        return this.careDeviceService.save(careDevice);
    }

    /**
     * 查询设备详情
     * @param id
     * @return
     */
    public DeviceVO getDeviceInfo(Long id){
        CareDevice careDevice = this.careDeviceService.getById(id);
        DeviceVO vo = new DeviceVO();
        BeanUtil.copyProperties(careDevice,vo);
        CareSysUser chamb = this.careSysUserService.getById(vo.getChambId());
        if(chamb != null) {
            vo.setChambName(chamb.getName());
            vo.setChambPhone(chamb.getPhone());
        }
        vo.setStationName(this.careStationService.getById(vo.getStationId()).getShortName());
        vo.setActiveStatusName(DeviceActiveStatusEnum.getCodeToName(vo.getActiveStatus()));
        vo.setStatusName(DeviceStatusEnum.getCodeToName(vo.getStatus()));

        if(StringUtils.isNotEmpty(vo.getDevScene())){
            vo.setIsHasScene(true);
        } else {
            vo.setIsHasScene(false);
        }

        if((vo.getFallingbedResponseTime() != null && vo.getFallingbedResponseTime() > 0)
               || (vo.getFallResponseTime() != null && vo.getFallResponseTime() > 0)
                || (vo.getLonglagResponseTime() != null && vo.getLonglagResponseTime() > 0)){
            vo.setIsHasResponseTime(true);
        } else {
            vo.setIsHasResponseTime(false);
        }
        if(CollUtil.isNotEmpty(queryMyOlderListByDeviceId(id))){
            vo.setIsHasOlder(true);
        } else {
            vo.setIsHasOlder(false);
        }
        if(CollUtil.isNotEmpty(queryMyContactListByDeviceId(id))){
            vo.setIsHasConnector(true);
        } else {
            vo.setIsHasConnector(false);
        }

        return vo;
    }
    /**
     * 查询服务站简介
     * @param stationId
     * @return
     */
    public StationVO getMyDevStationInfo(Long stationId){
        CareStation careStation = this.careStationService.getById(stationId);
        StationVO vo = new StationVO();
        BeanUtil.copyProperties(careStation,vo);
        return vo;
    }

    /**
     * 修改设备
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateMyDevice(DeviceVO vo){
        //修改设备
        CareDevice careDevice = this.careDeviceService.getById(vo.getId());
        BeanUtil.copyProperties(vo,careDevice);
        careDevice.setModifyTime(new Date());
        return this.careDeviceService.updateById(careDevice);
    }

    /**
     * 查询我的设备拥有的被监护人列表
     * @param devId
     * @return
     */
    public List<OlderVO> queryMyOlderListByDeviceId(Long devId){
        List<OlderVO> vos = Lists.newArrayList();
        CareDevice careDevice = this.careDeviceService.getById(devId);
        if(careDevice == null){
            return vos;
        }
        QueryWrapper<CareHouseOlderRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareHouseOlderRel::getHouseId, careDevice.getHouseId());
        List<CareHouseOlderRel> rels = this.careHouseOlderRelService.list(queryWrapper);
        if (CollUtil.isNotEmpty(rels)){
            rels.forEach(item -> {
                OlderVO vo = new OlderVO();
                CareOlder careOlder = this.careOlderService.getById(item.getOlderId());
                BeanUtil.copyProperties(careOlder,vo);
                vo.setLiveTypeDesc(OlderLiveTypeEnum.getCodeToName(vo.getLiveType()));
                vos.add(vo);
            });
        }
        return vos;
    }


    /**
     * 查询我的设备拥有的紧急联系人列表
     * @param devId
     * @return
     */
    public List<HouseContactVO> queryMyContactListByDeviceId(Long devId){
        List<HouseContactVO> vos = Lists.newArrayList();
        CareDevice careDevice = this.careDeviceService.getById(devId);
        if(careDevice == null){
            return vos;
        }
        QueryWrapper<CareHouseContactRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareHouseContactRel::getHouseId, careDevice.getHouseId());
        List<CareHouseContactRel> rels = this.careHouseContactRelService.list(queryWrapper);
        if (CollUtil.isNotEmpty(rels)){
            rels.forEach(item -> {
                HouseContactVO vo = new HouseContactVO();
                CareHouseContact careHouseContact = this.careHouseContactService.getById(item.getContactId());
                BeanUtil.copyProperties(careHouseContact,vo);
                vo.setRelationTypeDesc(RelationTypeEnum.getCodeToName(vo.getRelationType()));
                vos.add(vo);
            });
        }
        List<HouseContactVO> vosNew = CollUtil.sortByProperty(vos,"contactLevel");
        return vosNew;
    }

    /**
     * 绑定设备 和 老人
     * @return
     */
    public boolean bindOlder(Long devId,Long olderId){
        CareDevice careDevice = this.careDeviceService.getById(devId);
        CareHouseOlderRel careHouseOlderRel = new CareHouseOlderRel();
        careHouseOlderRel.setHouseId(careDevice.getHouseId());
        careHouseOlderRel.setOlderId(olderId);
        return this.careHouseOlderRelService.save(careHouseOlderRel);
    }

    /**
     * 移除被监护人，解绑设备 和 老人
     * @return
     */
    public boolean unBindOlder(Long devId,Long olderId){
        CareDevice careDevice = this.careDeviceService.getById(devId);

        QueryWrapper<CareHouseOlderRel> delWrapper = new QueryWrapper<>();
        delWrapper.lambda().eq(CareHouseOlderRel::getHouseId, careDevice.getHouseId())
        .eq(CareHouseOlderRel::getOlderId, olderId);
        return this.careHouseOlderRelService.remove(delWrapper);
    }

    /**
     * 绑定设备 和 紧急联系人
     * @return
     */
    public boolean bindHouseContact(Long devId,Long contactId){
        CareDevice careDevice = this.careDeviceService.getById(devId);
        CareHouseContactRel careHouseContactRel = new CareHouseContactRel();
        careHouseContactRel.setHouseId(careDevice.getHouseId());
        careHouseContactRel.setContactId(contactId);
        return this.careHouseContactRelService.save(careHouseContactRel);
    }

    /**
     * 移除被监护人，解绑设备 和 老人
     * @return
     */
    public boolean unHouseContact(Long devId,Long contactId){
        CareDevice careDevice = this.careDeviceService.getById(devId);

        QueryWrapper<CareHouseContactRel> delWrapper = new QueryWrapper<>();
        delWrapper.lambda().eq(CareHouseContactRel::getHouseId, careDevice.getHouseId())
                .eq(CareHouseContactRel::getContactId, contactId);
        return this.careHouseContactRelService.remove(delWrapper);
    }
}
