package com.care.common.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.care.common.entity.*;
import com.care.common.enums.ContactorStatusEnum;
import com.care.common.enums.UserRoleEnum;
import com.care.common.mapper.CcCallResultMapper;
import com.care.common.service.*;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.outcall.CcCallResultVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 外呼结果表(CcCallResult)表服务实现类
 *
 * @author makejava
 * @since 2021-05-30 13:13:26
 */
@Service
public class CcCallResultServiceImpl extends ServiceImpl<CcCallResultMapper, CcCallResult> implements CcCallResultService {

    @Resource
    private CareEventOrderHandleHisService careEventOrderHandleHisService;
    @Resource
    private CareEventOrderOlderService careEventOrderOlderService;
    @Resource
    private CareEventOrderChambService careEventOrderChambService;
    @Resource
    private CareEventOrderContactStatusService careEventOrderContactStatusService;
    /**
     * 获取呼叫结果
     *
     * @param id
     * @return
     */
    @Override
    public CcCallResultVO getCallResult(Long id) {
        return this.baseMapper.getCallResult(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveCallResult(CcCallResult result, UserLogindConvertVO loginUser) {

        this.save(result);

        CareEventOrderHandleHis his = new CareEventOrderHandleHis();
        his.setOrgId(result.getOrgId());
        his.setStationId(result.getStationId());
        his.setOrderId(result.getOrderId());
        // his.setLogType();
        his.setLogObjectId(result.getCalledId());
        his.setLogObjectName(result.getCalledName());
        his.setLogResult(ContactorStatusEnum.getCodeToName(result.getCallStatus()));
        his.setOpUserRole(UserRoleEnum.SEAT.getValue());
        his.setOpUserId(loginUser.getId());
        his.setOpUserName(loginUser.getName());
        his.setCreateTime(DateUtil.date());
        his.setRelationTypeDesc(result.getRelationTypeDesc());
        his.setCallResultId(result.getId());
        this.careEventOrderHandleHisService.save(his);

        if (UserRoleEnum.OLDER.getValue().equals(result.getCalledType())){
            UpdateWrapper<CareEventOrderOlder> updateWrapper = new UpdateWrapper<>();
            //已联系
            updateWrapper.lambda().eq(CareEventOrderOlder::getId,result.getCalledId())
            .set(CareEventOrderOlder::getStatus,"1");
            this.careEventOrderOlderService.update(updateWrapper);
        }else if (UserRoleEnum.CONTACT.getValue().equals(result.getCalledType())){
            UpdateWrapper<CareEventOrderContactStatus> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CareEventOrderContactStatus::getId,result.getCalledId())
                    .set(CareEventOrderContactStatus::getStatus,result.getCallStatus());
            this.careEventOrderContactStatusService.update(updateWrapper);
        }else if(UserRoleEnum.CHANMB.getValue().equals(result.getCalledType())){
            UpdateWrapper<CareEventOrderChamb> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(CareEventOrderChamb::getId,result.getCalledId())
                    .set(CareEventOrderChamb::getStatus,"1");
            this.careEventOrderChambService.update(updateWrapper);
        }

    }


}