package com.care.common.util;

import com.care.common.vo.PageResVO;
import org.apache.commons.lang3.StringUtils;


/**
 * @author nhc
 * @version 1.0
 * @description 带有分页的结果类
 * @date 2021/1/2 20:41
 */
public class PageResult<T> extends Result<T> {

    private PageResVO page;

    public PageResult(ResultCode resultCode){
        super(resultCode,null);
    }
    public PageResult(ResultCode resultCode, T data) {
        super(resultCode,data);
    }
    public PageResult(ResultCode resultCode,String message, T data) {
        super(resultCode,message,data);
    }
    public PageResult(ResultCode resultCode, String message, T data, PageResVO page) {
        super(resultCode,message,data);
        this.page = page;
    }
    public static<T> PageResult<T> success(String message,T data,long current, long pageSize, long total){
        PageResVO pageVo = new PageResVO(current,pageSize,total);
        PageResult<T> pageResult = new PageResult<>(ResultCode.SUCCESS);
        pageResult.setMessage(message);
        pageResult.setData(data);
        pageResult.setPage(pageVo);
        return pageResult;
    }

    public static<T> PageResult<T> success(T data,long current, long pageSize, long total){
        return success(ResultCode.SUCCESS.getMessage(),data,current,pageSize,total);
    }

    public static<T> PageResult<T> error(){
        return  error(ResultCode.ERROR.getMessage());
    }

    public static<T> PageResult<T> error(String message){
        PageResult<T> result  = new PageResult<>(ResultCode.ERROR);
        if(StringUtils.isNotEmpty(message)){
            result.setMessage(message);
        }
        return result;
    }

    public PageResVO getPage() {
        return page;
    }

    public void setPage(PageResVO page) {
        this.page = page;
    }
}
