package com.care.common.util;


import java.util.List;

/**
 * 分页工具类
 * @author javadoc
 */
public class PageUtil<T> {
    /**
     * 当前页码
     */
    private long current;
    /**
     * 页码大小（默认为10）
     */
    private long pageSize = 10;
    /**
     * 记录总数
     */
    private long total;
    /**
     * 总页数
     */
    private long pageCount;
    /**
     * 开始查询项
     */
    private long start;
    /**
     * 分页条目集合数据
     */
    private List<T> listT;
    /**
     * 分页条目单个数据
     */
    private T t;


    public long getCurrent() {
        return current;
    }

    /**
     * set 进当前页码时对越界进行修正,修正后计算出开始查询项
     * 注：每次在set进当前页前，一定要先更新总记录数，然后会顺便更新总页数，这样当前页的set才有效；
     * @param current 返回规范当前页
     */
    public void setCurrent(long current) {
        if(current < 1){
            current = 1;
        }
        if(pageCount != 0 && current > pageCount){
            current = pageCount;
        }
        this.current = current;
        this.start = (current - 1) * pageSize;
    }

    public long getPageSize() {
        return pageSize;
    }

    public void setPageSize(long pageSize) {
        this.pageSize = pageSize;
    }

    public long getTotal() {
        return total;
    }

    /**
     * set记录总数时自动计算总页数（大于取整+1）
     */
    public void setTotal(long total) {
        this.total = total;
        this.pageCount = (this.total % this.pageSize == 0) ? (this.total / this.pageSize) : (this.total / this.pageSize + 1);
    }

    public long getPageCount() {
        return pageCount;
    }

    public void setPageCount(long pageCount) {
        this.pageCount = pageCount;
    }

    public long getStart() {
        return start;
    }

    public void setStart(long start) {
        this.start = start;
    }

    public List<T> getListT() {
        return listT;
    }

    public void setListT(List<T> listT) {
        this.listT = listT;
    }

    public T getT() {
        return t;
    }

    public void setT(T t) {
        this.t = t;
    }

    @Override
    public String toString() {
        return "PageUtil{" +
                "current=" + current +
                ", pageSize=" + pageSize +
                ", total=" + total +
                ", pageCount=" + pageCount +
                ", start=" + start +
                ", listT=" + listT +
                ", t=" + t +
                '}';
    }
}
