package com.care.common.util;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.StringUtils;

/**
 * @author little
 */
@ApiModel(description = "请求返回实体类", value = "返回值")
@Setter
@Getter
public class Result<T> {
	@ApiModelProperty(dataType = "String", name = "code", value = "请求返回状态码")
	private String status;
	@ApiModelProperty(dataType = "String", name = "message", value = "请求返回描述信息")
	private String message;
	@ApiModelProperty(dataType = "T", name = "data", value = "返回数据")

	T data;

	public Result() {
		super();
	}
	public Result(ResultCode resultCode){
		this(resultCode,null);
	}
	public Result(ResultCode resultCode, T data) {
		this.status = resultCode.getStatus();
		this.message = resultCode.getMessage();
		this.data = data;
	}
	public Result(ResultCode resultCode,String message, T data) {
		this.status = resultCode.getStatus();
		this.message = message;
		this.data = data;
	}


	public static<T> Result<T> success(){
		return  success(ResultCode.SUCCESS.getMessage());
	}
	public static<T> Result<T> success(String message){
		return success(message,null);
	}
	public static<T> Result<T> success(T data){
		return success(ResultCode.SUCCESS.getMessage(),data);
	}
	public static<T> Result<T>success(String message,T data){
		Result<T> result  = new Result<>(ResultCode.SUCCESS);
		if(StringUtils.isNotEmpty(message)){
			result.setMessage(message);
		}
		if(data!=null){
			result.setData(data);
		}
		return result;
	}

	public static<T> Result<T> error(){
		return  error(ResultCode.ERROR.getMessage());
	}

	public static<T> Result<T> error(String message){
		Result<T> result  = new Result<>(ResultCode.ERROR);
		if(StringUtils.isNotEmpty(message)){
			result.setMessage(message);
		}
		return result;
	}

	public T getData() {
		return data;
	}
}
