package com.care.common.util;

import com.care.common.constant.Constants;
import com.care.common.vo.UserLogindConvertVO;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by y on 2017/11/23.
 * @author y
 */
public class WebPageUtils {

    public static final String IS_IPV4_ADDR = "^([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5]).([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5]).([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5]).([1-9]?\\d|1\\d\\d|2[0-4]\\d|25[0-5])$";

    /**
     * 获取当前登录用户
     * @param request
     * @return
     */
    public static UserLogindConvertVO getCurrentLoginedUser(HttpServletRequest request) {
        if (request == null) {
            return null;
        }
        Object obj = request.getAttribute(Constants.CURRENT_LOGINED_USER_KEY);
        return obj != null ? (UserLogindConvertVO)obj : null;
    }

    /**
     * 真实IP地址验证， 防止IP攻击，错误的IP导致安全问题
     */
    private static boolean isRealIPAddr(String ipAddr) throws Exception {
        boolean b1 = false;
        String spIP1 = "0.0.0.0";
        String spIp2 = "255.255.255.255";
        /**
         * 是否正确的ip地址
         */
        boolean b = regexValid(ipAddr, IS_IPV4_ADDR);
        if (b) {
            if (!spIP1.equals(ipAddr) && !spIp2.equals(ipAddr)) {
                b1 = true;
            }
        }
        return b1;
    }

    /**
     * 验证方法
     * @param input
     * @param reg
     * @return
     * @throws Exception
     */
    public static boolean regexValid(String input, String reg) throws Exception {
        if (input == null || "".equals(input) || reg == null || "".equals(reg)) {
            throw new Exception("参数不能为空!");
        }
        Pattern regex = Pattern.compile(reg);
        Matcher matcher = regex.matcher(input);
        return matcher.matches();
    }

    /**
     * 获取客户端真实ip地址
     */
    public static String getIpAddr(HttpServletRequest request) throws Exception {
        boolean b = false;
        String ip = request.getHeader("x-forwarded-for");
        String unKnown = "unknown";
        if (StringUtils.isNotBlank(ip) && !unKnown.equalsIgnoreCase(ip)) {
            /**
             * 多次反向代理后会有多个ip值，第一个ip才是真实ip
             */
            int index = ip.indexOf(",");
            if (index != -1) {
                ip = ip.substring(0, index);
                b = isRealIPAddr(ip);
                if (!b) {
                    ip = "";
                }
            }
        }
        if (ip == null || ip.length() == 0 || unKnown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Real-IP");
            if (StringUtils.isNotBlank(ip)) {
                b = isRealIPAddr(ip);
                if (!b) {
                    ip = "";
                }
            }
        }
        if (ip == null || ip.length() == 0 || unKnown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
            if (StringUtils.isNotBlank(ip)) {
                b = isRealIPAddr(ip);
                if (!b) {
                    ip = "";
                }
            }
        }
        if (ip == null || ip.length() == 0 || unKnown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
            if (StringUtils.isNotBlank(ip)) {
                b = isRealIPAddr(ip);
                if (!b) {
                    ip = "";
                }
            }
        }
        if (ip == null || ip.length() == 0 || unKnown.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    public static void main(String[] args) {
        System.out.println(DigestUtils.md5Hex("123456"));
    }
}
