package com.care.installation.controller;


import com.care.common.util.Result;
import com.care.installation.service.DevInstallService;

import com.care.installation.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * 描述：
 *
 * @autho：wenskys
 * @create 2020/5/12 14:13
 **/
@Api(tags = "设备安装接口")
@RestController
@RequestMapping("/installation/api")
public class DevInstallController {

    @Resource
    private DevInstallService devInstallService;



    /**
     * 验证设备（是否存在）
     * @param devCode
     * @return
     */
    @PostMapping("checkDevCode")
    @ApiOperation(tags = {"设备安装"},value = "验证设备")
    public Result<Object> checkDevCode(@RequestParam("devCode") String devCode){
        if (this.devInstallService.checkDevCode(devCode)){
            return  Result.success("验证成功");
        } else{
            return  Result.error("验证失败");
        }
    }

    /**
     * 获取设备安装信息（地址（经纬度）、设备标签、场景）
     * @param devCode
     * @return
     */
    @GetMapping("getDevInstallationInfo")
    @ApiOperation(tags = {"设备安装"},value = "获取设备安装信息（地址（经纬度）、设备标签、场景）")
    public Result<DeviceInstallationVO> getDevInstallationInfo(@RequestParam("devCode") String devCode) {
       return Result.success(this.devInstallService.getDevInstallationInfo(devCode));
    }


    /**
     * 保存设备安装信息
     * @return
     */
    @PostMapping("saveDevInstallation")
    @ApiOperation(tags = {"设备安装"},value = "保存设备安装信息（地址（经纬度）、设备标签、设备场景，注意：devCode必须传")
    public Result<Object> saveDevInstallation(@RequestBody DeviceInstallationVO vo){
        if (this.devInstallService.saveDevInstallation(vo)){
            return  Result.success("保存成功");
        } else{
            return  Result.error("保存失败");
        }
    }


}
