package com.care.keeper.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.care.common.entity.*;
import com.care.common.service.*;
import com.care.common.util.Result;
import com.care.keeper.service.AbstractKeeperPassportService;
import com.care.keeper.service.KeeperInfoApiService;
import com.care.keeper.service.KeeperPassportService;
import com.care.keeper.vo.KeeperInfoVO;
import com.care.keeper.vo.OlderVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 描述：
 *
 * @autho：wenskys
 * @create 2020/5/12 14:13
 **/
@Api(tags = "用户相关接口")
@RestController
@RequestMapping("/keeper/api/member")
public class KeeperMemberApiController {

    @Resource
    private CareSysUserService careSysUserService;

    @Resource
    private KeeperPassportService keeperPassportService;

    @Resource
    private AbstractKeeperPassportService abstractKeeperPassportService;

    @Resource
    private CareStationService careStationService;
    @Resource
    private KeeperInfoApiService keeperInfoApiService;



    @ApiOperation(value = "获取基本信息", notes = "获取用户基本录信息 -- 二期修改")
    @GetMapping("detail/info")
    public Result<Object> detailInfo(@RequestHeader String token) {
        KeeperInfoVO current = keeperPassportService.checkToken(token);
        CareSysUser member = careSysUserService.getById(current.getId());
        BeanUtil.copyProperties(member,current);
        if(current.getStationId() != null) {
            current.setStationName(careStationService.getById(current.getStationId()).getName());
        }
        //服务人数
        current.setServeCount(this.keeperInfoApiService.queryOlderCount(current.getId()));

        //老人列表
        List<OlderVO> olderVOList =  keeperInfoApiService.getOlderList(current.getId());
        if(CollUtil.isNotEmpty(olderVOList)){
            current.setOlderVOList(olderVOList);
        }

        return Result.success(current);
    }

    @PostMapping("/update/updateMyInfo")
    @ApiOperation(value = "修改个人信息（电话、姓名、头像、性别等） -- 二期新增", notes = "修改个人信息（电话、姓名、头像、性别等）")
    public Result<Object> updateMyInfo(@RequestHeader("token") String token,
                                             @RequestBody KeeperInfoVO vo){
        try {
            KeeperInfoVO current = keeperPassportService.checkToken(token);
            if(current == null ){
                return Result.error("修改失败！");
            }
            vo.setId(current.getId());
            if(this.keeperInfoApiService.updateMyInfo(vo)){
                return  Result.success("修改成功！");
            } else {
                return Result.error("修改失败！");
            }

        } catch (Exception e) {
            return Result.error("修改失败！");
        }

    }

    @ApiOperation(value = "刷新基本信息", notes = "刷新用户基本录信息")
    @PostMapping("refresh")
    public Result<Object> refreshMember(@RequestHeader String token) {
        KeeperInfoVO lastMemberInfoVO = abstractKeeperPassportService.refreshMember(token);
        return Result.success(lastMemberInfoVO);
    }

   /* @ApiOperation(value = "校验是否要刷新小程序用户信息", notes = "校验是否要刷新小程序用户信息 true刷新")
    @PostMapping("notMatchMemberField")
    public Result<Object> notMatchMemberField(@RequestHeader String token) {
        return Result.success(passportService.notMatchMemberField(token));
    }
*/

}
