package com.care.mqtt.service;


import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.entity.CareDevice;
import com.care.common.enums.*;
import com.care.common.service.CareDeviceService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;


/**
 * 使用redis轮训方案实现延迟触发
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttHearbeatService {
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private RedisUtil redisUtil;


    /**
     * 读取数据库设备表，激活的，轮训看redis中是否有心跳消息，有则置为在线，没有置为下线
     */
    public void rotationDev() {

        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getActiveStatus, DeviceActiveStatusEnum.ACTIVE.getValue());

        List<CareDevice> careDeviceList = careDeviceService.list(queryWrapper);
        if (CollUtil.isNotEmpty(careDeviceList)) {
            List<CareDevice> careDeviceListNew = new ArrayList<>();
            for (CareDevice careDevice : careDeviceList) {
                CareDevice careDeviceNew = new CareDevice();
                careDeviceNew.setId(careDevice.getId());

                if (redisUtil.hget(RedisKeyConstant.MQTT_DEV_HEARBEAT_INFO,careDevice.getTopic()) != null) { //在线
                    if(!DeviceStatusEnum.ONLINE.getValue().equals(careDevice.getStatus())){
                        careDeviceNew.setStatus(DeviceStatusEnum.ONLINE.getValue());
                        careDeviceListNew.add(careDeviceNew);
                    }
                } else { //离线
                    if(!DeviceStatusEnum.OFFLINE.getValue().equals(careDevice.getStatus())){
                        careDeviceNew.setStatus(DeviceStatusEnum.OFFLINE.getValue());
                        careDeviceListNew.add(careDeviceNew);
                    }
                }
            }

            if (CollUtil.isNotEmpty(careDeviceListNew)){
                careDeviceService.updateBatchById(careDeviceListNew);
            }

        }
    }

}
