package com.care.mqtt.service;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.care.bigscreen.service.BigScreenService;

import com.care.common.cache.RedisKeyConstant;
import com.care.common.cache.RedisUtil;
import com.care.common.entity.*;

import com.care.common.enums.MqttMsgStatusEnum;
import com.care.common.enums.OrderStatusEnum;
import com.care.common.enums.OrderTypeEnum;
import com.care.common.service.*;
import com.care.common.vo.device.MqttMsgVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

import java.util.Map;
import java.util.UUID;

/**
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttMsgService {
    @Resource
    private CareHouseService careHouseService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private CareOrganizationService careOrganizationService;
    @Resource
    private BigScreenService bigScreenService;
    @Resource
    private CareMqttMsgService careMqttMsgService;
    @Resource
    private MqttMsgRedisService mqttMsgRedisService;
    @Resource
    private RedisUtil redisUtil;


    private static Logger logger = LoggerFactory.getLogger(MqttMsgService.class);

    /**
     * 处理mqtt消息
     * @param topic
     * @param mqttMessageStr
     * @throws Exception
     */
    public void handleMessage(String topic, String mqttMessageStr) throws Exception {
        handleOrderEventMessage(topic,mqttMessageStr);
    }

        /**
         * 工单事件的消息 处理
         * @param topic
         * @param mqttMessageStr
         */
    private void handleOrderEventMessage(String topic, String mqttMessageStr) {

        try {
            Map map = JSON.parseObject(mqttMessageStr);
            String type = (String) map.get("type");

            if(StringUtils.isEmpty(type)){
                logger.error("不是正确格式的消息");
                return;
            }
            //处理心跳,放入redis
            if ("HeartBeat".equals(type)) {
                logger.info("{} 接收到MQTT  HeartBeat消息,{}",topic,mqttMessageStr);
                redisUtil.hset(RedisKeyConstant.MQTT_DEV_HEARBEAT_INFO, topic, "1", RedisKeyConstant.MQTT_DEV_HEARBEAT_INFO_TIME);
                return;
            }

            if(!"BreathHeartRate".equals(type) && !"FallDown".equals(type) && !"StayTooLong".equals(type)){
               // logger.error("不是要处理的消息");
                return;
            }
            logger.info("{} 接收到MQTT 事件消息,{}",topic,mqttMessageStr);
            CareDevice  careDevice = getCareDevice(topic);
            if(careDevice == null) {
                logger.error("数据库中没找到对应设备");
                return;
            }

            Date date = DateUtil.date();

            CareMqttMsg careMqttMsg = new CareMqttMsg();
            careMqttMsg.setMqttMessage(mqttMessageStr);
            careMqttMsg.setTopic(topic);
            careMqttMsg.setDevId(careDevice.getId());
            careMqttMsg.setType(type);
            careMqttMsg.setCreateTime(date);
            Map msg = (Map) map.get("msg");
            Integer tid = (Integer) msg.get("tid");
            careMqttMsg.setTid(tid);

            if ("BreathHeartRate".equals(type)) {
                careMqttMsg.setStatus(MqttMsgStatusEnum.HANDLED.getValue());
                this.careMqttMsgService.save(careMqttMsg);

                Integer breath = (Integer) msg.get("breath");
                Integer heart = (Integer) msg.get("heart");
                careDevice.setBreathRate(breath);
                careDevice.setHeartRate(heart);
                careDeviceService.updateById(careDevice);
                return;
            }

            if( careDevice.getHouseId() == null ) {
                logger.error("设备没有配置房屋地址");
                return;
            }

            handleOrderEvent(map,type,careDevice,date,careMqttMsg);

        } catch (Exception e){
            e.printStackTrace();
        }
    }

    private void handleOrderEvent(Map map,String type,CareDevice careDevice,Date date, CareMqttMsg careMqttMsg){
        String key = RedisKeyConstant.CREATE_ORDER + ":" + careDevice.getId();
        String requestId = UUID.randomUUID().toString();
        boolean result = redisUtil.tryLock(key,requestId,10);
        try {
            if (result) {
                CareEventOrder order = getNewOrder(careDevice);
                if (order == null) {
                    logger.error("设备没有配置房屋地址");
                    return;
                }
                order.setCreateTime(date);
                order.setModifyTime(date);

                Map msgMap = (Map) map.get("msg");
                Integer conf = (Integer) msgMap.get("conf");

                if (conf != 255) {
                    order.setStatus(OrderStatusEnum.TODO.getValue());
                } else {//取消
                    order.setStatus(OrderStatusEnum.CANCEL.getValue());
                }

                if ("FallDown".equals(type)) { //跌到
                    order.setOrderType(OrderTypeEnum.DIE_DAO.getValue());
                } else if ("StayTooLong".equals(type)) {
                    order.setOrderType(OrderTypeEnum.JIU_ZHI.getValue());
                }
                order.setTid(careMqttMsg.getTid());

                //判断房屋是否有未完成的告警事件,区分类型和人
                QueryWrapper<CareEventOrder> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.lambda().eq(CareEventOrder::getDevId, careDevice.getId())
                        .eq(CareEventOrder::getOrderType,order.getOrderType())
                        .eq(CareEventOrder::getTid,order.getTid())
                        .and(
                        wrapper -> wrapper.eq(CareEventOrder::getStatus, OrderStatusEnum.TODO.getValue())
                                .or().eq(CareEventOrder::getStatus, OrderStatusEnum.DOING.getValue())
                        );
                CareEventOrder orderDb = careEventOrderService.getOne(queryWrapper2);

                if (orderDb != null) { //有未完成的告警事件，不再生成新的告警工单，只作为子事件插入到当前工单的历史记录中。
                    if (conf != 255) {
                        careMqttMsg.setStatus(MqttMsgStatusEnum.HANDLED.getValue());
                        this.careMqttMsgService.save(careMqttMsg);

                        careEventOrderService.saveHisOrder(order, orderDb,false);
                        //通知页面
                        this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "orderUpdate");
                        this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "bigscreenRefresh");
                    } else { //取消
                        careMqttMsg.setStatus(MqttMsgStatusEnum.HANDLED.getValue());
                        this.careMqttMsgService.save(careMqttMsg);

                        //更新当前的工单状态为取消
                        order.setId(orderDb.getId());
                        this.careEventOrderService.autoCancelOrder(order);
                        //通知页面
                        this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "orderUpdate");
                        this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "bigscreenRefresh");

                    }

                } else { //无未完成的告警事件
                    Integer delayTime = getDelayTime(type, careDevice);
                    if (delayTime == null || delayTime == 0) { //没有配置响应时间，立即生成新的告警事件工单
                        if (conf != 255) {
                            careMqttMsg.setStatus(MqttMsgStatusEnum.HANDLED.getValue());
                            this.careMqttMsgService.save(careMqttMsg);

                            //生成新的工单
                            this.careEventOrderService.saveOrder(order);

                            //通知页面
                            this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "orderAdd");
                            this.bigScreenService.pushRtEventFlag(order.getOrgId(),order.getStationId(), order.getId().toString(), "bigscreenRefresh");
                        } else { //取消 不处理工单
                            careMqttMsg.setStatus(MqttMsgStatusEnum.CANCEL.getValue());
                            this.careMqttMsgService.save(careMqttMsg);
                        }

                    } else { //有配置响应时间，不处理工单，进行调度处理
                        if (conf != 255) { //不是取消
                            careMqttMsg.setStatus(MqttMsgStatusEnum.NO_HANDLE.getValue());
                            careMqttMsg.setDelayTime(delayTime);
                            this.careMqttMsgService.save(careMqttMsg);

                            //进行调度处理
                            MqttMsgVO mqttMsgVO = new MqttMsgVO();
                            BeanUtil.copyProperties(careMqttMsg, mqttMsgVO);
                            mqttMsgRedisService.addRedis(mqttMsgVO, date.getTime() + careMqttMsg.getDelayTime() * 60 * 1000);
                        } else { //取消： 更新mqtt消息为已取消，等待执行的调度到时会取消执行
                            careMqttMsg.setStatus(MqttMsgStatusEnum.CANCEL.getValue());
                            this.careMqttMsgService.save(careMqttMsg);

                            //更新mqtt消息为已取消
                            UpdateWrapper<CareMqttMsg> updateWrapper = new UpdateWrapper<>();
                            updateWrapper.lambda().eq(CareMqttMsg::getDevId, careDevice.getId()).eq(CareMqttMsg::getType, type).eq(CareMqttMsg::getStatus, MqttMsgStatusEnum.NO_HANDLE.getValue())
                                    .set(CareMqttMsg::getStatus, MqttMsgStatusEnum.CANCEL.getValue());
                            this.careMqttMsgService.update(updateWrapper);
                        }
                    }
                }
            } else {
                try {
                    Thread.sleep(3000);
                    handleOrderEvent(map,type,careDevice,date,careMqttMsg);
                } catch (Exception e){
                    e.printStackTrace();
                }

            }
        }  finally {
            redisUtil.releaseLock(key,requestId);
        }

    }

    private CareDevice getCareDevice(String topic){
        QueryWrapper<CareDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareDevice::getTopic,topic);
        CareDevice careDevice = careDeviceService.getOne(queryWrapper);
        return careDevice;
    }

    private Integer getDelayTime(String type, CareDevice careDevice) {
        if ("FallDown".equals(type)) { //跌到
            return careDevice.getFallResponseTime();
        } else if ("StayTooLong".equals(type)) { //久滞
            return careDevice.getLonglagResponseTime();
        } else {
            return 0;
        }
    }

  public CareEventOrder getNewOrder(CareDevice careDevice) {
      CareHouse careHouse = careHouseService.getById(careDevice.getHouseId());
      if (careHouse == null){
          logger.error("设备没有配置房屋地址");
          return null;
      }
      CareEventOrder order = new CareEventOrder();
      order.setOrgId(careDevice.getOrgId());
      order.setOrgName(careOrganizationService.getById(careDevice.getOrgId()).getName());
      order.setStationId(careDevice.getStationId());
      order.setStationName(careStationService.getById(careDevice.getStationId()).getName());
      order.setHouseId(careHouse.getId());
      order.setDevId(careDevice.getId());
      order.setDevCode(careDevice.getDevCode());
      order.setDevName(careDevice.getDevName());
      order.setHouseName(careHouse.getName());
      order.setMemberId(careDevice.getMemberId());
      if(careHouse.getHouseNumber() != null ){
          order.setTitle(careHouse.getAddr() + careHouse.getHouseNumber());
      } else {
          order.setTitle(careHouse.getAddr());
      }

      return order;
  }

}
