package com.care.outcall.service.impl;


import com.care.outcall.entity.CcKoalaLog;
import com.care.outcall.mapper.KoalaOutCallDao;
import com.care.outcall.service.KoalaOutCallService;
import com.care.common.util.DateUtils;
import com.care.common.util.JsonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author stw
 * @version 2.7.0 创建于 2019/7/15
 **/
@Service
public class KoalaOutCallServiceImpl implements KoalaOutCallService {

    @Autowired
    private KoalaOutCallDao koalaOutCallDao;

    @Override
    public void addOptLog(CcKoalaLog ccKoalaLog) throws Exception {
        koalaOutCallDao.addOptLog(makeLog(ccKoalaLog));
    }

    private static Map<String, Object> parseJson(String json) throws Exception {
        return JsonUtil.fromJson(json, Map.class);
    }

    private CcKoalaLog makeLog(CcKoalaLog ccKoalaLog) throws Exception {
        //前端调用接口即触发日志记录事件
        CcKoalaLog domain = new CcKoalaLog();
        domain.setResult(ccKoalaLog.getResult());
        domain.setApiMethod(ccKoalaLog.getApiMethod());
        domain.setComputeTraffic("0");//话单未被计算

        domain.setAgentId(ccKoalaLog.getAgentId());
        domain.setOrgId(ccKoalaLog.getOrgId());
        domain.setStationId(ccKoalaLog.getStationId());
        domain.setOlderId(ccKoalaLog.getOlderId());
        domain.setOrderId(ccKoalaLog.getOrderId());
        domain.setCalledId(ccKoalaLog.getCalledId());
        domain.setHouseId(ccKoalaLog.getHouseId());
        domain.setCalledName(ccKoalaLog.getCalledName());
        domain.setRelationTypeDesc(ccKoalaLog.getRelationTypeDesc());
        domain.setContactLevel(ccKoalaLog.getContactLevel());
        domain.setCalledType(ccKoalaLog.getCalledType());
        domain.setOutDisplayNo(ccKoalaLog.getOutDisplayNo());

        //解析请求结果，获取sessionId和坐席分机号等
        Map<String, Object> map = parseJson(ccKoalaLog.getResult());
        Object sessionId = map.get("callID");
        Object agentNumber = map.get("thisDN");
        Object receiveTime = map.get("creationTime");
        Object tenantId = map.get("tenantID");
        Object messageId = map.get("messageId");
        Object callState = map.get("callState");//0：被叫挂断，1：主叫挂断
        Object phone = map.get("otherDN");//被叫号码
        if(sessionId != null){
            domain.setSessionId(sessionId.toString());
        }
        if(agentNumber != null){
            domain.setAgentNumber(agentNumber.toString());
        }
        if(receiveTime != null){
            domain.setReceiveTime(DateUtils.formatMillisecondTime(receiveTime.toString()));
        }
        if(tenantId != null){
            domain.setTenantId(tenantId.toString());
        }
        if(messageId != null){
            //根据考拉外呼messageId确定事件状态
            //1.坐席振铃,2.坐席接通,3.客户振铃,4.客户接通,5.坐席挂断,6.客户挂断,7.未接通'
            //callState为空，默认认为是被叫挂断（用于挂断状态处理）
            domain.setEventStatus(formatOutCallEvent(messageId.toString(), (callState != null) ? callState.toString() : "1"));
        }
        if(phone != null){
            domain.setPhone(phone.toString());
        }
        return domain;
    }

    /**
     * 根据考拉外呼messageId确定事件状态
     * @param messageId
     * @return
     */
    private String formatOutCallEvent(String messageId, String callState){
        //1.坐席振铃,2.坐席接通,3.客户振铃,4.客户接通,5.坐席挂断,6.客户挂断,7.未接通'
        String eventStatus = "";
        switch (messageId){
            case "505":
                eventStatus = "1";
                break;
            case "506":
                eventStatus = "4";
                break;
            case "515":
                eventStatus = "6";
                if("1".equals(callState)){
                    eventStatus = "5";
                }
                break;
            default:
                eventStatus = "7";
                break;
        }
        return eventStatus;
    }

    @Override
    public Map<String, String> getAgentInfo(String userId) throws Exception {

        Map<String, String> koalaAccountNoAndApiSecret = this.koalaOutCallDao.getAgentInfo(userId);

        String entId = koalaAccountNoAndApiSecret.get("entId");
        String agentId = koalaAccountNoAndApiSecret.get("agentId");
        String agentPassword = koalaAccountNoAndApiSecret.get("agentPassword");

        Map<String, String> map = new HashMap<>();
        map.put("entId", entId);
        map.put("agentId", agentId);
        map.put("agentPassword", agentPassword);
        return map;
    }

    /**
     * 获取sessionId
     * @param ccKoalaLog
     * @return
     * @throws Exception
     */
    @Override
    public String getSessionId(CcKoalaLog ccKoalaLog) throws Exception{
        return this.koalaOutCallDao.getSessionId(ccKoalaLog);
    }
}
